/*****************************************************************************
 *  analyze.c: here are functions for analyze tested IC from analyze file or
 *  		from database
 *  This is part of IC-tester program.
 *  Copyright (C) 2001,2002  Silvio Klaic, sklaic@student.math.hr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *****************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#ifndef DOS
#include <unistd.h>
#else
#include <dos.h>
#endif

#include "portlib.h"
#include "device.h"
#include "find.h"
#include "convert.h"

short int analyze_ic(unsigned int IO, char *F_name)
{
char ch[256],tmp[20];
int lastpr,i;
unsigned long end_file,pos_file;
short int testing=2,ErroR=0;
FILE *f;

if(F_name==NULL){ /* no file name write error */
 printf("No analyze data file specified, can not continue!\n");
 return(0);/* return error */
}/* end if file name exists */
if((f=find_A_file(F_name,".adf"))==NULL){/* file not opened */
 printf("Can't access analyze data file %s: %s !\n",F_name,strerror(errno));
 return(0);/* return error */
}else{/* all ok continue */
 if (allow_io(IO, 2) == -1) {/* test port access */
  fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);
 }else{/* ok access granted continue */
  if(is_power_on(IO)) printf("\n => WARNING: ictester device not powered - can not do check !\n\n");
  else{/* all ok do check */
   enable_port(1,IO);lastpr=-1;/* enable ictester device */
   fseek(f,0L,SEEK_END);end_file=ftell(f);rewind(f);/* get file size */
   fgets(ch,80,f);/*ch[strlen(ch)-1]=0;*//* get first line */
   if(strcmp(ch,"Analyze IC Tester data\n")==0){/* yeah this is it... */
    printf("Selected port is: 0x%03X. Information from analyze file (%ld bytes):\n",IO,end_file);
    do{/* go thru file */
     ch[0]=0;fgets(ch,256,f);ch[strlen(ch)-1]=0;/* read line by line */
     if( (ch[0]!=0)&&(ch[0]!='#')&&(ch[0]!=32)&&(ch[0]!='\n')&&(ch[1]==':') ){
      /* data line */
      switch (ch[0]){/* find type of information */
       case 'A':/* author entry */
	      ch[0]='r';printf("Autho%s\n",ch);break;
       case 'M':/* date entry */
	      ch[0]='e';printf("Last modify dat%s\n",ch);break;
       case 'D':/* description entry */
	      ch[0]='n';printf("IC descriptio%s\n",ch);break;
       case 'I':/* not tested entry */
	      ch[0]='n';printf("Test may work o%s\n",ch);break;
       case 'T':/* tested entry */ 
	      ch[0]='n';printf("Tested o%s\n",ch);break;
       case 'P':/* paused entry */
	      for(i=0;i<=19;i++) tmp[i]=ch[i+3];
#ifndef DOS
#ifndef __minix
	      usleep(atol(tmp)*1000);
#endif
#else
	      delay(atol(tmp));
#endif
	      break;/* delay */
       case 'W':/* write entry */
	      if(testing==2){/* ok we read info, ask for continue to test */
	       printf("Press ENTER key to begin test, Q for quit... ");
	       fgets(tmp,80,stdin);/* get char */
	       if((tmp[0]=='q')||(tmp[0]=='Q')) testing=0; else testing=1;
	      }/* end if testing==2 */
	      if(testing==1){/* perform test */
	       if(ErroR==3) ErroR=1;/* reset error */
	       for(i=0;i<=15;i++) tmp[i]=ch[i+3];/* copy test data */
               if(portwrite(IO,tmp)){/* setting data fail, abort */
                printf("\r%79s\rError! Data aren't successful write to device.\
\nThere is problem with transferring data to device, possible low power?\n"," ");
                testing=0;}/* end if portwrite, abort testing */
	      }break;/* end if testing==1 */
       case 'R':/* read entry */
	      if(testing==1){/* perform test */
	       if(ErroR==3) ErroR=1;/* reset error */
               portread(IO,tmp);tmp[16]=0;/* read data */
	       if(strcmp(tmp,"================")==0){/* woha, read error */
		printf("\r%79s\rError! Read data from device are corrupted.\
\n"," ");testing=0;/* abort testing */
	       }else{/* ok compare data */
		for(i=0;i<=15;i++) if( (ch[i+3]!='=')&&(tmp[i]!=ch[i+3]) )
		 ErroR=2;/* error in compare */
	       }/* end else compare data */
	      }break;
       case 'E':/* error entry */
	      if((testing==1)&&(ErroR>=2)){/* write error */
	       if(ErroR==2){
	        ch[0]='r';printf("\r%79s\rErro%s\n"," ",ch);ErroR=3;
	       }else{
	        ch[0]=' ';ch[1]=' ';printf("\r%79s\r%s\n"," ",ch);
	       }
	      }break;
       case '?':/* error input entry */
	      if((testing==1)&&ErroR){/* write quit message */
	       ch[0]='=';ch[1]='>';printf("\r%79s\r%s\n"," ",ch);
	       printf("Press ENTER key to continue, Q for quit... ");
	       fgets(tmp,20,stdin);/* get char */
	       if((tmp[0]=='q')||(tmp[0]=='Q')) testing=0; else testing=1;
	      }break;
       default:/* unknown entry */
	      printf("Hm? Unknown entry '%c'!\nFull line: %s\n",ch[0],ch);break;
      }/* end switch */
      if(testing==1){/* ok write progress bar */
       pos_file=ftell(f);if(lastpr!=div(pos_file*100,end_file).quot){
       sprintf(ch,"\rTesting: |%s",/* ok progress changes, do update */
	"====================================================");
       ch[(short int)ldiv(51*pos_file,end_file).quot+11]=0;
       lastpr=div(pos_file*100,end_file).quot;printf("\r%-62s| %3d%%",ch,
	lastpr);fflush(NULL);}/* progress bar */
      }/* end if testing==1 */
     }/* end if is data line */
    }while(!feof(f)&&(testing!=0));/* end go thru file */
    if(ErroR) printf("\r%79s\rTesting fail, tested IC don't work.\n"," ");
    else if(testing!=0) printf("\r%79s\rTesting complete. Tested IC is 100%%\
 correct.\n"," ");/* write ok message */
   }else if(strcmp(ch,"Analyze IC Tester link\n")==0){/* yeah this is link... */
     ch[0]=0;fgets(ch,256,f);ch[strlen(ch)-1]=0;/* read information line */
     printf("%s\n",ch);ch[0]=0;fgets(ch,256,f);ch[strlen(ch)-1]=0;/* location */
     analyze_ic(IO,ch);ErroR=-1;/* do new scan with other file */
    }else{/* mh? this isn't IC Tester analyze file, abort check! */
     printf("File %s isn't IC Tester analyze file!\n",F_name);/* write error */
    }/* end else if strcmp */
   if(ErroR!=-1) enable_port(0,IO);/* reset and disable ictester device */
  }/* end else if powered */
  if (disallow_io(IO, 2) == -1)/* close port */
   fprintf(stderr, "Woha! Can't disengage port.\n\
You could have some problems with this port from now on...\n");
 }/* end else if port access */
 fclose(f);/* close file */
}/* end else if fopen */
return(1);/* return all ok */
}/* end analyze_ic() */

short int analyze_ic_data(unsigned int IO, char *F_name)
{
char ch[256],tmp[20];
int lastpr,i;
unsigned long end_file,pos_file;
short int testing=2,ErroR=0;
FILE *f;

if((f=find_A_file(F_name,".adf"))==NULL){/* file not opened */
 printf("Can't access analyze data file %s: %s !\n",F_name,strerror(errno));
 return(0);/* return error */
}else{/* all ok continue */
 fseek(f,0L,SEEK_END);end_file=ftell(f);rewind(f);/* get file size */
 fgets(ch,80,f);/*ch[strlen(ch)-1]=0;*//* get first line */
 if(strcmp(ch,"Analyze IC Tester data\n")==0){/* yeah this is it... */
  lastpr=-1;do{/* go thru file */
   ch[0]=0;fgets(ch,256,f);ch[strlen(ch)-1]=0;/* read line by line */
   if( (ch[0]!=0)&&(ch[0]!='#')&&(ch[0]!=32)&&(ch[0]!='\n')&&(ch[1]==':') ){
   /* data line */
   switch (ch[0]){/* find type of information */
    case 'A':/* author entry skip */
    case 'D':/* description entry skip */
    case 'T':/* tested entry skip */ 
    case 'I':break;/* not tested entry skip */
    case 'M':/* date entry */
	      ch[0]='d';printf("Last modifie%s\n",ch);break;
    case 'P':/* paused entry */
	      for(i=0;i<=19;i++) tmp[i]=ch[i+3];
#ifndef DOS
#ifndef __minix
	      usleep(atol(tmp)*1000);
#endif
#else
	      delay(atol(tmp));
#endif
	      break;/* delay */
    case 'W':/* write entry */
	      if(testing==2){/* ok we read info, ask for continue to test */
	       printf("Press ENTER key to begin test, Q for quit... ");
	       fgets(tmp,80,stdin);/* get char */
	       if((tmp[0]=='q')||(tmp[0]=='Q')) testing=0; else testing=1;
	      }/* end if testing==2 */
	      if(testing==1){/* perform test */
	       if(ErroR==3) ErroR=1;/* reset error */
	       for(i=0;i<=15;i++) tmp[i]=ch[i+3];/* copy test data */
               if(portwrite(IO,tmp)){/* setting data fail, abort */
                printf("\r%79s\rError! Data aren't successful write to device.\
\nThere is problem with transferring data to device, possible low power?\n"," ");
                testing=0;}/* end if portwrite, abort testing */
	      }break;/* end if testing==1 */
    case 'R':/* read entry */
	      if(testing==1){/* perform test */
	       if(ErroR==3) ErroR=1;/* reset error */
               portread(IO,tmp);tmp[16]=0;/* read data */
	       if(strcmp(tmp,"================")==0){/* woha, read error */
		printf("\r%79s\rError! Read data from device are corrupted.\
\n"," ");testing=0;/* abort testing */
	       }else{/* ok compare data */
		for(i=0;i<=15;i++) if( (ch[i+3]!='=')&&(tmp[i]!=ch[i+3]) )
		 ErroR=2;/* error in compare */
	       }/* end else compare data */
	      }break;
    case 'E':/* error entry */
	      if((testing==1)&&(ErroR>=2)){/* write error */
	       if(ErroR==2){
	        ch[0]='r';printf("\r%79s\rErro%s\n"," ",ch);ErroR=3;
	       }else{
	        ch[0]=' ';ch[1]=' ';printf("\r%79s\r%s\n"," ",ch);
	       }
	      }break;
    case '?':/* error input entry */
	      if((testing==1)&&ErroR){/* write quit message */
	       ch[0]='=';ch[1]='>';printf("\r%79s\r%s\n"," ",ch);
	       printf("Press ENTER key to continue, Q for quit... ");
	       fgets(tmp,20,stdin);/* get char */
	       if((tmp[0]=='q')||(tmp[0]=='Q')) testing=0; else testing=1;
	      }break;
    default:/* unknown entry */
	      printf("Hm? Unknown entry '%c'!\nFull line: %s\n",ch[0],ch);break;
   }/* end switch */
   if(testing==1){/* ok write progress bar */
    pos_file=ftell(f);if(lastpr!=div(pos_file*100,end_file).quot){
    sprintf(ch,"\rTesting: |%s",/* ok progress changes, do update */
	"====================================================");
    ch[(short int)ldiv(51*pos_file,end_file).quot+11]=0;
    lastpr=div(pos_file*100,end_file).quot;printf("\r%-62s| %3d%%",ch,
	lastpr);fflush(NULL);}/* progress bar */
   }/* end if testing==1 */
   }/* end if is data line */
  }while(!feof(f)&&(testing!=0));/* end go thru file */
  if(ErroR) printf("\r%79s\rTesting fail, tested ic don't work.\n"," ");
  else if(testing!=0) printf("\r%79s\rTesting complete. Tested IC is 100%%\
 correct.\n"," ");/* write ok message */
 }else if(strcmp(ch,"Analyze IC Tester link\n")==0){/* this is another link?! */
  printf("Another link??? Aborting test, too many linked files.\n");
 }else{/* mh? this isn't IC Tester analyze file, abort check! */
  printf("File %s isn't IC Tester analyze file!\n",F_name);/* write error */
 }/* end else if strcmp */
 fclose(f);/* close file */
}/* end else if fopen */
return(1);/* return all ok */
}/* end analyze_ic() */

void danalyze_ic(unsigned int IO,unsigned long int PoS,char *D_loc)
{
FILE *f;
int i,ErroR,lastpr;
unsigned long int RestDat;
char ln[255],Lname[128];

if((f=fopen(D_loc,"rb"))==NULL)/* can't open database */
 fprintf(stderr, "Can't open database %s!\n",D_loc);
else{/* database opened, go to known position and do analyze */
 if (allow_io(IO, 2) == -1) {/* test port access */
  fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);fclose(f);/* close database */
 }else{/* ok access granted continue */
  if(is_power_on(IO)){ printf("\n => WARNING: ictester device not powered -\
 can not do check!\n\n");fclose(f);/* close database */
  }else{/* all ok do check */
   enable_port(1,IO);/* enable ictester device */
   fseek(f,(long)PoS,SEEK_SET);/* skip to data area */
   printf("Selected port is: 0x%03X. Information from database:\n",IO);
   ln[0]=0;i=fgetc(f);fgets(ln,(i&127)+1,f);/* get size and string id */
   if (i&128) {/* link entry read location */
    printf("For testing %s using ",ln);i=fgetc(f);Lname[0]=0;
    fgets(Lname,i+1,f);printf("%s test data.\n",Lname);/* read link id entry */
    fgetc(f);fgetc(f);fgetc(f);fgetc(f);/* skip data size */
    i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);/* get IC true name */
    if(ln[0]!=0) printf("%s:",ln);/* if exists write name */
    i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);/* get IC description */
    if(ln[0]!=0) printf(" %s\n",ln);/* if exists write description */
    printf("Tested: ");i=fgetc(f)*256;i+=fgetc(f);/* get size of tested ics */
    if(i<=0) printf("none."); else{/* check if there's data, else write it */
    for(;i>0;i--) printf("%c",fgetc(f));}/* write data */
    printf("\nAuthor(s): ");i=fgetc(f)*256;i+=fgetc(f);/* get size of authors */
    if(i<=0) printf("unknown."); else{/* check if there's data, else write it */
    for(;i>0;i--) printf("%c",fgetc(f));}/* write data */
    printf("\n");ln[0]=0;fclose(f);/* close database */
    PoS=find_D_file(D_loc,Lname);/* try to find entry in database */
    if(PoS!=0){/* do only test from database */
     if((f=fopen(D_loc,"rb"))==NULL)/* can't open database */
      fprintf(stderr, "Can't open database %s!\n",D_loc);
     else{/* database opened, go to known position and do analyze */
      fseek(f,(long)PoS,SEEK_SET);/* skip to data area */
      ln[0]=0;i=fgetc(f);fgets(ln,(i&127)+1,f);/* get size and string id */
      if (i&128)/* this is another link?! */
       printf("Another link??? Aborting test, too many linked files.\n");
      else{/* analyze entry in database, do read */
       RestDat=(long)fgetc(f)*256*256*256;RestDat+=(long)fgetc(f)*256*256;
       RestDat+=(long)fgetc(f)*256;RestDat+=(long)fgetc(f);RestDat+=ftell(f);
       /* get end position of data */
       i=fgetc(f);fseek(f,(long)i,SEEK_CUR);/* skip IC true name */
       i=fgetc(f);fseek(f,(long)i,SEEK_CUR);/* skip IC description */
       i=fgetc(f)*256;i+=fgetc(f);fseek(f,(long)i,SEEK_CUR);/* skipp test ics */
       i=fgetc(f)*256;i+=fgetc(f);fseek(f,(long)i,SEEK_CUR);/* skipp authors */
       printf("Last modified: ");ln[0]=0;fgets(ln,21,f);/* get date */
       printf("%s\n",ln);/* write date */
       fseek(f,(long)fgetc(f)*256*256*256+fgetc(f)*256*256+fgetc(f)*256+
	fgetc(f), SEEK_CUR);/* skip to data area */
       printf("Press ENTER key to begin test, Q for quit... ");
       fgets(ln,80,stdin);/* get char */
       if((ln[0]!='q')&&(ln[0]!='Q')){ ErroR=0;lastpr=-1;/* do test */
        while(ftell(f)<RestDat){/* got thru data to end */ 
         switch(fgetc(f)){/* scan what action is taken */
          case 0:/* write data to device */
              if(ErroR==3) ErroR=1;/* if there was error,set ErroR to 1 not 0 */
	      ln[0]=fgetc(f);ln[1]=fgetc(f);ln[2]=fgetc(f);ln[3]=fgetc(f);
	      dofrom4(ln);if(portwrite(IO,ln)){/* setting data fail, abort */
	       printf("\r%79s\rError! Data aren't successful write to device.\
\nThere is problem with transferring data to device, possible low power?\n"," ");
	      RestDat=0;}/* end if portwrite, abort testing */
	      break;/* end erite data */
          case 1:/* read data from device */
              if(ErroR==3) ErroR=1;/* if there was error,set ErroR to 1 not 0 */
	      ln[0]=fgetc(f);ln[1]=fgetc(f);ln[2]=fgetc(f);ln[3]=fgetc(f);
	      dofrom4(ln);portread(IO,Lname);Lname[16]=0;/* read data */
	      if(strcmp(Lname,"================")==0){/* woha, read error */
	       printf("\r%79s\rError! Read data from device are corrupted.\
\n"," ");RestDat=0;/* abort testing */
	      }else{/* ok compare data */
               for(i=0;i<=15;i++) if( (ln[i]!='=')&&(Lname[i]!=ln[i]) )
                ErroR=2;/* error in compare, mark for error display */
              }/* end else compare data */
	      break;/* end read data */
          case 2:/* error report */
	      i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);if(ErroR>=2){/* Error */
	       if(ErroR==2){/* first error report write Error on begin */
		printf("\r%79s\rError: %s\n"," ",ln);ErroR=3;
	       }else printf("\r%79s\r%s\n"," ",ln);/* write other error lines */
	      }/* end if ErroR */
	      break;
          case 3:/* ask report */
	      i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);if(ErroR){/* error detected */
	       printf("\r%79s\r%s\n"," ",ln);/* write message */
	       printf("Press ENTER key to continue, Q for quit... ");
	       fgets(ln,80,stdin);/* get char */
	       if((ln[0]=='q')||(ln[0]=='Q')) RestDat=0;/* quit */
	      } break;/* end if ErroR */
          case 4:/* pause */
	      i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);/* get value */
#ifndef DOS
#ifndef __minix
	      usleep(atol(ln)*1000);
#endif
#else
              delay(atol(ln));
#endif
	      break;/* pause - delay */
          default:/* unknown action */
	      printf("Kme? Unknown action in database at byte %ld.\nCan not\
 continue, possible error or new type of database.\nTry to solve error in\
 database or get new version of program.\n",ftell(f)-1);RestDat=0;break;
         }/* end switch thru actions */
         if(lastpr!=(int)ldiv((ftell(f)-PoS)*100,RestDat-PoS).quot){/* if new */
          sprintf(ln,"\rTesting: |============================================\
========");/* end indicator */
	  ln[(short int)ldiv(51*(ftell(f)-PoS),RestDat-PoS).quot+11]=0;
          lastpr=(int)ldiv((ftell(f)-PoS)*100,RestDat-PoS).quot;/* percent */
          printf("\r%-62s| %3d%%",ln,lastpr);fflush(NULL);/* write progress */
         }/* end if there are changes in progress percentage */
        }/* end while thru end */
        if(ErroR) printf("\r%79s\rTesting fail, tested IC don't work.\n"," ");
        else if(RestDat!=0) printf("\r%79s\rTesting complete. Tested IC is\
 100%% correct.\n"," ");/* write pass ok message */
       }/* end if do test */
      }/* end if analyze entry in database */
      fclose(f);
     }/* end if database opened */
    }else if(!analyze_ic_data(IO,Lname))/* check analyze files */
     fprintf(stderr, "Requested entry '%s' is not found under any database or\
 analyze file!\n",Lname);
   }else{/* analyze entry in database, do read */
    RestDat=(long)fgetc(f)*256*256*256;RestDat+=(long)fgetc(f)*256*256;
    RestDat+=(long)fgetc(f)*256;RestDat+=(long)fgetc(f);RestDat+=ftell(f);
    /* get end position of data */
    i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);/* get IC true name */
    if(ln[0]!=0) printf("%s:",ln);/* if exists write name */
    i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);/* get IC description */
    if(ln[0]!=0) printf(" %s\n",ln);/* if exists write description */
    printf("Tested: ");i=fgetc(f)*256;i+=fgetc(f);/* get size of tested ICs */
    if(i<=0) printf("none."); else{/* check if there's data, else write it */
    for(;i>0;i--) printf("%c",fgetc(f));}/* write data */
    printf("\nAuthor(s): ");i=fgetc(f)*256;i+=fgetc(f);/* get size of authors */
    if(i<=0) printf("unknown."); else{/* check if there's data, else write it */
    for(;i>0;i--) printf("%c",fgetc(f));}/* write data */
    printf("\nLast modified: ");ln[0]=0;fgets(ln,21,f);/* get date */
    printf("%s\n",ln);/* write date */
    fseek(f,(long)fgetc(f)*256*256*256+fgetc(f)*256*256+fgetc(f)*256+fgetc(f),
	SEEK_CUR);/* skip to data area */
    printf("Press ENTER key to begin test, Q for quit... ");fgets(ln,80,stdin);
    if((ln[0]!='q')&&(ln[0]!='Q')){ ErroR=0;lastpr=-1;/* do test */
     while(ftell(f)<RestDat){/* got thru data to end */ 
      switch(fgetc(f)){/* scan what action is taken */
       case 0:/* write data to device */
              if(ErroR==3) ErroR=1;/* if there was error,set ErroR to 1 not 0 */
	      ln[0]=fgetc(f);ln[1]=fgetc(f);ln[2]=fgetc(f);ln[3]=fgetc(f);
	      dofrom4(ln);if(portwrite(IO,ln)){/* setting data fail, abort */
	       printf("\r%79s\rError! Data aren't successful write to device.\
\nThere is problem with transferring data to device, possible low power?\n"," ");
	      RestDat=0;}/* end if portwrite, abort testing */
	      break;/* end erite data */
       case 1:/* read data from device */
              if(ErroR==3) ErroR=1;/* if there was error,set ErroR to 1 not 0 */
	      ln[0]=fgetc(f);ln[1]=fgetc(f);ln[2]=fgetc(f);ln[3]=fgetc(f);
	      dofrom4(ln);portread(IO,Lname);Lname[16]=0;/* read data */
	      if(strcmp(Lname,"================")==0){/* woha, read error */
	       printf("\r%79s\rError! Read data from device are corrupted.\
\n"," ");RestDat=0;/* abort testing */
	      }else{/* ok compare data */
               for(i=0;i<=15;i++) if( (ln[i]!='=')&&(Lname[i]!=ln[i]) )
                ErroR=2;/* error in compare, mark for error display */
              }/* end else compare data */
	      break;/* end read data */
       case 2:/* error report */
	      i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);if(ErroR>=2){/* Error */
	       if(ErroR==2){/* first error report write Error on begin */
		printf("\r%79s\rError: %s\n"," ",ln);ErroR=3;
	       }else printf("\r%79s\r%s\n"," ",ln);/* write other error lines */
	      }/* end if ErroR */
	      break;
       case 3:/* ask report */
	      i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);if(ErroR){/* error detected */
	       printf("\r%79s\r%s\n"," ",ln);/* write message */
	       printf("Press ENTER key to continue, Q for quit... ");
	       fgets(ln,80,stdin);/* get char */
	       if((ln[0]=='q')||(ln[0]=='Q')) RestDat=0;/* quit */
	      } break;/* end if ErroR */
       case 4:/* pause */
	      i=fgetc(f);ln[0]=0;fgets(ln,i+1,f);/* get value */
#ifndef DOS
#ifndef __minix
	      usleep(atol(ln)*1000);
#endif
#else
              delay(atol(ln));
#endif
	      break;/* pause - delay */
       default:/* unknown action */
	      printf("Kme? Unknown action in database at byte %ld.\nCan not\
 continue, possible error or new type of database.\nTry to solve error in\
 database or get new version of program.\n",ftell(f)-1);RestDat=0;break;
      }/* end switch thru actions */
      if(lastpr!=(int)ldiv((ftell(f)-PoS)*100,RestDat-PoS).quot){/* new perc */
       sprintf(ln,"\rTesting: |==============================================\
======");/* end indicator */
       ln[(short int)ldiv(51*(ftell(f)-PoS),RestDat-PoS).quot+11]=0;
       lastpr=(int)ldiv((ftell(f)-PoS)*100,RestDat-PoS).quot;/* get percent */
       printf("\r%-62s| %3d%%",ln,lastpr);fflush(NULL);/* write progress bar */
      }/* end if there are changes in progress percentage */
     }/* end while thru end */
     if(ErroR) printf("\r%79s\rTesting fail, tested IC don't work.\n"," ");
     else if(RestDat!=0) printf("\r%79s\rTesting complete. Tested IC is 100%%\
 correct.\n"," ");/* write pass ok message */
    }/* end if do test */
    fclose(f);/* close database */
   }/* end else if link entry */
   enable_port(0,IO);/* reset and disable ictester device */
  }/* end else if powered */
  if (disallow_io(IO, 2) == -1)/* close port */
   fprintf(stderr, "Woha! Can't disengage port.\n\
You could have some problems with this port from now on...\n");
 }/* end else if port access */
}/* end else if open database */
}/* end danalyze_ic() */

