/*****************************************************************************
 *  device.c: here are functions for working with ictester device
 *  This is part of IC-tester program.
 *  Copyright (C) 2001,2002  Silvio Klaic, sklaic@student.math.hr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *****************************************************************************/

#ifndef DOS
#include <unistd.h>
#endif
#include <stdio.h>
#include <string.h>

#include "portlib.h"
#include "device.h"

int is_power_on( unsigned int IO )/* test if is device connected */
{
unsigned int rs=0;

 rs=read_byte(IO+1);/* read current status of port */
 if (rs & 16) return(0);/* all ok */
 else return(1);/* 5V not present */
}/* end is_power_on() */

short int portwrite( unsigned int IO, const char DAT[16] )
{
int i,ret=0;
short int table[32];
unsigned int pw,pr;

/* Writing to test socket: (format: table bit, pin on socket, type of set)
* 1,9,-  : 2,9,+   : 3,10,+  : 4,10,-  : 5,11,-  : 6,11,+  : 7,12,+  : 8,12,-
* 9,13,- : 10,13,+ : 11,14,+ : 12,14,- : 13,15,- : 14,15,+ : 15,16,+ : 16,16,-
* 17,1,- : 18,1,+  : 19,2,+  : 20,2,-  : 21,3,-  : 22,3,+  : 23,4,+  : 24,4,-
* 25,5,- : 26,5,+  : 27,6,+  : 28,6,-  : 29,7,-  : 30,7,+  : 31,8,+  : 32,8,-
* (Type of set: '+' = write (+5V): '-' = read (GND) */

 for(i=0;i<=15;i++){/* go thru data for writing */
  switch (DAT[i]) {/* check type of writing */
   case '1':/* pin enable, set true (+5V) */
	   switch (i){/* set each bit to right place */
	    case 0: table[17]=1;table[16]=0;break;/*  set bit 1 to +5V */
	    case 1: table[18]=1;table[19]=0;break;/*  set bit 2 to +5V */
	    case 2: table[21]=1;table[20]=0;break;/*  set bit 3 to +5V */
	    case 3: table[22]=1;table[23]=0;break;/*  set bit 4 to +5V */
	    case 4: table[25]=1;table[24]=0;break;/*  set bit 5 to +5V */
	    case 5: table[26]=1;table[27]=0;break;/*  set bit 6 to +5V */
	    case 6: table[29]=1;table[28]=0;break;/*  set bit 7 to +5V */
	    case 7: table[30]=1;table[31]=0;break;/*  set bit 8 to +5V */
	    case 8: table[1]=1;table[0]=0;break;/*    set bit 9 to +5V */
	    case 9: table[2]=1;table[3]=0;break;/*    set bit 10 to +5V */
	    case 10: table[5]=1;table[4]=0;break;/*   set bit 11 to +5V */
	    case 11: table[6]=1;table[7]=0;break;/*   set bit 12 to +5V */
	    case 12: table[9]=1;table[8]=0;break;/*   set bit 13 to +5V */
	    case 13: table[10]=1;table[11]=0;break;/* set bit 14 to +5V */
	    case 14: table[13]=1;table[12]=0;break;/* set bit 15 to +5V */
	    case 15: table[14]=1;table[15]=0;break;/* set bit 16 to +5V */
	    default:/* eh? Malfunction? */
		     fprintf(stderr,"\r...kmeeee... ...kmeeee... ...kmeeee...\n\
Error when setting bit on socket for +5V! Request was for setting bit %d.\n\
=> Possible memory garbage, try compile without -O2 flag.\n",i);break;
	   }break;/* end switch */
   case '0':/* pin enable, set false (GND) */
	   switch (i){/* set each bit to right place */
	    case 0: table[17]=0;table[16]=1;break;/*  set bit 1 to GND */
	    case 1: table[18]=0;table[19]=1;break;/*  set bit 2 to GND */
	    case 2: table[21]=0;table[20]=1;break;/*  set bit 3 to GND */
	    case 3: table[22]=0;table[23]=1;break;/*  set bit 4 to GND */
	    case 4: table[25]=0;table[24]=1;break;/*  set bit 5 to GND */
	    case 5: table[26]=0;table[27]=1;break;/*  set bit 6 to GND */
	    case 6: table[29]=0;table[28]=1;break;/*  set bit 7 to GND */
	    case 7: table[30]=0;table[31]=1;break;/*  set bit 8 to GND */
	    case 8: table[1]=0;table[0]=1;break;/*    set bit 9 to GND */
	    case 9: table[2]=0;table[3]=1;break;/*    set bit 10 to GND */
	    case 10: table[5]=0;table[4]=1;break;/*   set bit 11 to GND */
	    case 11: table[6]=0;table[7]=1;break;/*   set bit 12 to GND */
	    case 12: table[9]=0;table[8]=1;break;/*  set bit 13 to GND */
	    case 13: table[10]=0;table[11]=1;break;/*  set bit 14 to GND */
	    case 14: table[13]=0;table[12]=1;break;/* set bit 15 to GND */
	    case 15: table[14]=0;table[15]=1;break;/* set bit 16 to GND */
	    default:/* eh? malfunction? */
		     fprintf(stderr,"\r...kmeeee... ...kmeeee... ...kmeeee...\n\
Error when setting bit on socket for GND! Request was for setting bit %d.\n\
=> Possible memory garbage, try compile without -O2 flag.\n",i);break;
	   }break;/* end switch */
   default:/* pin disable, set not active (N/A) */
	   switch (i){/* set each bit to right place */
	    case 0: table[17]=0;table[16]=0;break;/*  set bit 1 to N/A */
	    case 1: table[18]=0;table[19]=0;break;/*  set bit 2 to N/A */
	    case 2: table[21]=0;table[20]=0;break;/*  set bit 3 to N/A */
	    case 3: table[22]=0;table[23]=0;break;/*  set bit 4 to N/A */
	    case 4: table[25]=0;table[24]=0;break;/*  set bit 5 to N/A */
	    case 5: table[26]=0;table[27]=0;break;/*  set bit 6 to N/A */
	    case 6: table[29]=0;table[28]=0;break;/*  set bit 7 to N/A */
	    case 7: table[30]=0;table[31]=0;break;/*  set bit 8 to N/A */
	    case 8: table[1]=0;table[0]=0;break;/*    set bit 9 to N/A */
	    case 9: table[2]=0;table[3]=0;break;/*    set bit 10 to N/A */
	    case 10: table[5]=0;table[4]=0;break;/*   set bit 11 to N/A */
	    case 11: table[6]=0;table[7]=0;break;/*   set bit 12 to N/A */
	    case 12: table[9]=0;table[8]=0;break;/*  set bit 13 to N/A */
	    case 13: table[10]=0;table[11]=0;break;/*  set bit 14 to N/A */
	    case 14: table[13]=0;table[12]=0;break;/* set bit 15 to N/A */
	    case 15: table[14]=0;table[15]=0;break;/* set bit 16 to N/A */
	    default:/* eh? malfunction? */
		     fprintf(stderr,"\r...kmeeee... ...kmeeee... ...kmeeee...\n\
Error when setting bit on socket for N/A! Request was for setting bit %d.\n\
=> Possible memory garbage, try compile without -O2 flag.\n",i);break;
	   }break;/* end switch */
  }/* end switch */
 }/* end for i */
 pw=read_byte(IO);/* read current status at port */
 pw=(pw & 239);/* set strobe to lock */
 pw=(pw & 223) | 32;/* set clock to hold */
 write_byte(pw,IO);/* write new status */
 for(i=0;i<=31;i++){/* go thru data and write it to port */
  pw=(pw & 191) | (table[i]*64);/* set new parameters for data */
  pw=(pw & 223) | 32;/* set clock to hold */
  write_byte(pw,IO);/* write new status */
  pw=(pw & 223) | 0;/* set clock to next */
  write_byte(pw,IO);/* write new status */
  pw=(pw & 223) | 32;/* set clock to hold */
  write_byte(pw,IO);/* write new status */
 }/* end for i */
 for(i=0;i<=31;i++){/* go thru data again and check write data */
  pr=read_byte(IO+1);/* read current status at port */
  if (pr & 32) pr=1;else pr=0;/* check data */
  if (pr!=table[i]) ret++;/* data not match */
  pw=(pw & 191) | (table[i]*64);/* set new parameters for data */
  pw=(pw & 223) | 32;/* set clock to hold */
  write_byte(pw,IO);/* write new status */
  pw=(pw & 223) | 0;/* set clock to next */
  write_byte(pw,IO);/* write new status */
  pw=(pw & 223) | 32;/* set clock to hold */
  write_byte(pw,IO);/* write new status */
 }/* end for i */
 if(!ret){/* ok data set successful do write */
  pw=(pw & 239) | 16;/* set strobe to unlock */
  write_byte(pw,IO);/* write new status */
  pw=(pw & 239);/* set strobe to lock */
  write_byte(pw,IO);/* write new status */
 }/* end if !ret */
 return(ret);
}/* end portwrite() */

void portread( unsigned int IO, char *DAT )
{
int i;
char D1[20],D2[20],D3[20];
unsigned int pw,pr;

/* We do here double - triple read, in that way there's fixed problem with
 * short connection. E.g.: on output of tested IC is HIGH (+5V) and tester give
 * GND signal, this is short circuit, but tester will adapt this power in
 * few milliseconds, that time matches first reading cycle.
 * Next reading should get stable status, if differ from first, make third
 * reading to confirm reading.
 * Make delay here? No, it's better this way, because when reading, IC1 rise
 * input power from 0.5 to 0.7 volts to get better sensitive input, which in
 * some cases change power state at test socket. */
/* read first time */
for(i=0;i<=15;i++){/* send signal for reading each pin */
 pw=read_byte(IO);/* read current status at port */
 pw=(pw & 240) | i;/* set new parameters for writing */
 write_byte(pw,IO);/* write new status */
 pr=read_byte(IO+1);/* read data */
 if(pr & 128) D1[i]='1'; else D1[i]='0';/* put data to buffer */
}/* end for i */
/* read second time */
for(i=0;i<=15;i++){/* send signal for reading each pin */
 pw=read_byte(IO);/* read current status at port */
 pw=(pw & 240) | i;/* set new parameters for writing */
 write_byte(pw,IO);/* write new status */
 pr=read_byte(IO+1);/* read data */
 if(pr & 128) D2[i]='1'; else D2[i]='0';/* put data to buffer */
}/* end for i */
D1[16]=0;D2[16]=0;/* make string and compare data */
if(strcmp(D1,D2)==0) for(i=0;i<=15;i++) DAT[i]=D1[i];/* found identical */
else{ /* first two reading not identically, read third time */
 for(i=0;i<=15;i++){/* send signal for reading each pin */
  pw=read_byte(IO);/* read current status at port */
  pw=(pw & 240) | i;/* set new parameters for writing */
  write_byte(pw,IO);/* write new status */
  pr=read_byte(IO+1);/* read data */
  if(pr & 128) D3[i]='1'; else D3[i]='0';/* put data to buffer */
 }/* end for i */
 /* compare reading, use 2 identical results for real */
 D3[16]=0;/* make string */
 if(strcmp(D1,D3)==0) for(i=0;i<=15;i++) DAT[i]=D1[i];/* found identical */
 else if(strcmp(D2,D3)==0) for(i=0;i<=15;i++) DAT[i]=D2[i];/* found identical */
  else {/* opa, there is wild reading, show error */
   for(i=0;i<=15;i++) DAT[i]='=';}
}/* end else if not match */
}/* end portread() */

void enable_port(short int WhaT, unsigned int IO)
{
unsigned int pw;

pw=read_byte(IO);/* read current status at port */
switch (WhaT){/* check what type of action to use */
 case 1:/* do enable and reset ictester device for writing */
       pw=(pw & 127) | 128;/* set enable to write */
       portwrite(IO,"================");break;/* reset device */
 case 2:/* disable ictester device for writing */
       pw=(pw & 127);break;/* set disable to write */
 case 3:/* do enable ictester device for writing */
       pw=(pw & 127) | 128;break;/* set enable to write */
 default:/* (including WhaT==0) disable and reset ictester device for writing */
       portwrite(IO,"================");/* reset device */
       pw=(pw & 127);break;/* set disable to write */
}/* end switch WhaT */
write_byte(pw,IO);/* write new status */
}/* end enable_port() */

