/*****************************************************************************
 *  main.c: here is main function for working with program
 *  This is part of IC-tester program.
 *  Copyright (C) 2001, 2002  Silvio Klaic, sklaic@student.math.hr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "test.h"
#include "test_ocr.h"
#include "rw.h"
#include "dump.h"
#include "analyze.h"
#include "find.h"
#include "diodeocr.h"

#ifdef __minix
_PROTOTYPE(void usage, (char *P));
_PROTOTYPE(int main, (int argc, char** argv));
#endif

void usage (char *P)
{
 fprintf(stderr,"Usage: %s -[p port_no] -[a[do]dfhilrst[o]v] -[w <data>]\
 -[ca <file>] [IC_number]\n\
\tIC_number\t: root number from tested IC, eq. 7400\n\
\t-a <file>\t: check IC from analyze file, analyze tested ic.\n\
\t-ad\t\t: DIODE test, with auto detect.\n\
\t-ao\t\t: OCR test (triac and thyristor), with auto detect.\n\
\t-c <file>\t: check IC from dump file, compare with data from file.\n\
\t-d\t\t: dump IC, use for dummy test and finding type of ic.\n\
\t-f\t\t: measure working frequency.\n\
\t-h\t\t: display this usage information.\n\
\t-l\t\t: lock write on ictester and reset device (safe mode).\n\
\t-p <port>\t: set communication to port <port> default 888 (0x378).\n\
\t-R\t\t: rapidly read data from device and report errors.\n\
\t-r\t\t: read data from test socket on ictester.\n\
\t-s\t\t: create list of data files in supported.html file.\n\
\t-t\t\t: test functionality of IC Tester device.\n\
\t-to\t\t: test functionality of OCR add-on on IC Tester device.\n\
\t-v\t\t: print the program version number.\n\
\t-w <data>\t: write (set) <data> to test socket, eq: -w 0==1==00==1\n\
    (First character is for pin 1 on test socket, second for pin 2, etc.\n\
    Use: '1' to set +5V, '0' to set 0V (GND), '=' to set not connected (NC) )\n\
For more detail about all options, use command: icthelp\n\
For updating database use command: icupdata\n",P);
}/* end usage() */

int main( int argc, char** argv )
{
int i;
char loc[256];
unsigned int port;
unsigned long int pos;

if (argc <= 1) { usage(argv[0]); exit (1); }

port=0x378;/* set default port */
for(i=1;i<argc;i++){/* go thru arguments if any*/
 if(argv[i][0]=='-'){/* found options */
  switch (argv[i][1]) {/* check type of option */
   case 'a': switch (argv[i][2]){/* check which part */
	      case 'd':diode_test(port);break;/* check diode */
	      case 'o':triac_test(port);break;/* check OCR */
	      default:analyze_ic(port,argv[i+1]);i++;
		      break;/* analyze ic from file */
	     } break;/* end if analyze, diode or ocr test */
   case 'c': check_ic(port,argv[i+1]);i++;break;/* compare ic from file */
   case 'd': dump_ic(port);break;/* dump data from ic */
   case 'f': freq_ic(port);break;/* measure speed */
   case 'h': usage(argv[0]);break;/* help */
   case 'l': lock_device(port);break;/* reset port and disable write */
   case 'p': port=atol(argv[i+1]);i++;break;/* port change */
   case 'R': R_test(port);break;/* rapidly read data */
   case 'r': r_test(port);break;/* read data */
   case 's': create_supported_file();break;/* create supported.html file */
   case 't': switch (argv[i][2]){/* check wich part */
	      case 'o':test_ocr(port);break;/* test OCR add-on */
	      default:test(port);break;/* test device functionality */
	     } break;/* end if test req. */
   case 'v': /* write version */
#ifdef DOS
fprintf(stderr, "Dos 16 bit program version: %s\n",VER);
#else
#ifdef __minix
fprintf(stderr, "Minix 16 bit program version: %s\n",VER);
#else
fprintf(stderr, "Linux program version: %s\n",VER);
#endif
#endif
fprintf(stderr, "(C) Copyright Silvio Klaic 2001,2002 - this program is under\
 GPL licence!\n");break;/* version */
   case 'w': w_test(port,argv[i+1]);i++;break;/* write data */
   default : fprintf(stderr, "Warning: unknown option '%s' !\n",argv[i]);break;
  }/* end of switch */
 }else{ /* check input for database request */
  pos=find_D_file(loc,argv[i]);/* try to find entry in database */
  if(pos!=0) danalyze_ic(port,pos,loc);/* scan from database */
  else if(!analyze_ic(port,argv[i]))/* not in database, check analyze files */
    fprintf(stderr, "Requested entry '%s' is not found under any database or\
 analyze file!\nPossible wrong typed argument?\n",argv[i]);
 }/* end else if input */
}/* end for i */
 return 0;
}/* end main() */

