/**********************************************************************-*-C-*-
 *
 * portlib.c - This file was part of `xradio', now is part of `ictester'
 *
 * Copyright (C) 1996 Pat Thoyts (thoyts@bsa.bristol.ac.uk)
 *
 * New changes, Copyright (C) 2001,2002 Silvio Klaic (sklaic@student.math.hr)
 *
 * This file provides all the routines used to send and receive data to and
 * from the ports. To work, this file MUST be compiled with -O2 and NOT with
 * -g and the resulting executable has to be suid root.
 *
 *****************************************************************************/

#ifndef DOS
#include <unistd.h>
#ifdef __minix
#include <fcntl.h>
#else
#include <sys/io.h>
#endif
#else
#include <dos.h>
#include <io.h>
#endif

#include "portlib.h"

/*
 * allow_io() and disallow_io() use the system call `ioperm' to enable use
 * of low numbered ports. These functions insist upon root access.
 *
 * We also look at /dev/console and see who owns it and whether it matches our
 * real user id (our euid is root) using access("/dev/console", R_OK);
 *
 * Locking? Nah? Maybe some idiot will run two copies - what happens?
 */
int allow_io( unsigned long base, unsigned long num )
{
int tmp=0;

#if !defined DOS && !defined __minix
if ( geteuid() != 0
#if !defined(NO_CHECK_CONSOLE)
       || access("/dev/console", R_OK) != 0
#endif
       )
  return -1;
  tmp = ioperm(base, num, 1);
#else
#ifdef __minix
/* here we need access to /dev/mem, otherwise program can't access ports */
tmp=open("/dev/mem",O_RDONLY);
if(tmp < 0)/* uh no access to memory??? */
 fprintf(stderr,"Couldn't open /dev/mem, can't access to port.\n");
else{ outb(base+2,0);/* set bidirectional port to out */
 close(tmp);tmp=0;}/* close memory */
#endif
#endif
return tmp;
}/* end allow_io() */

int disallow_io( unsigned long base, unsigned long num )
{
#if !defined DOS && !defined __minix
int tmp;

if ( geteuid() != 0
#if !defined(NO_CHECK_CONSOLE)
       || access("/dev/console", R_OK) != 0
#endif
       )
    return -1;

  tmp = ioperm(base, num, 0);
  return tmp;
#else
  return (0);
#endif
}/* end disallow_io() */

/*
 * If we have port access, read or write 8 or 16 bit values from the ports
 * opened.
 */
unsigned int read_byte ( unsigned int port )
{
  unsigned int value = 0;
#ifndef DOS
  value = inb( port );
#else
__asm {
		 mov dx, port
		 in  al,dx
		 mov Byte(value),al
		}
#endif
  return value;
}/* end read_byte() */

unsigned int read_word ( unsigned int port )
{
  unsigned int value = 0;
#ifndef DOS
  value = (inb(port + 1)<<8);
  value &= inb(port);
#else
  unsigned int p, v = 0;
p=port+1;
__asm {
		 mov dx, p
		 in  al,dx
		 mov Byte(value),al
		}
__asm {
		 mov dx, port
		 in  al,dx
		 mov Byte(v),al
		}
value = value<<8;
value &= v;
#endif
  return value;
}/* end read_word() */

void write_byte ( unsigned int bytes, unsigned int port )
{
  bytes &= 0x00FF;
#ifndef DOS
#ifdef __minix
  outb( port, bytes );
#else
  outb( bytes, port );
#endif
#else
__asm {
		 mov dx, port
		 mov al, Byte(bytes)
		 out dx,al
		}
#endif
#ifdef DEBUG_PORTLIB
  fprintf(stderr,"PORTLIB: writing 8 bit value 0x%02X to 0x%03X\n", byte, port);
#endif
}/* end write_byte() */

void write_word ( unsigned long word, unsigned int port )
{
  unsigned int lo = 0;
  lo = (word & 0x00FF);
#ifndef DOS
#ifndef __minix
  outw( lo, port );
#endif
#else
/* unsigned int hi = 0; */
/*  hi = ((word & 0xFF00)>>8);*/
/* need to fix this to work with word not byte */
/* need to put minix word write */
__asm {
		 mov dx, port
		 mov al, Byte(lo)
		 out dx,al
		}
#endif
#ifdef DEBUG_PORTLIB
  fprintf(stderr, "PORTLIB: writing 16 bit value 0x%04lx to 0x%03x\n",
	  word, port);
#endif
}/* end write_word() */

/* Read data from ports from `base' to `base+num' and output on stream.
 * Basically for debugging purposes.  */
void print_ports ( FILE* stream, unsigned long base, unsigned long num )
{
  unsigned long cn = 0;

  fprintf(stream, "Ports:");
  for (cn = 0; cn < num; cn++) {
	 fprintf(stream, " %04lX:%0X", base + cn, read_byte(base + cn));
  }
  fprintf(stream, "\n");
}/* end print_ports() */

