/*****************************************************************************
 *  rw.c: here are functions for manual setting and reading data from
 *  	ictester device and for locking it.
 *  This is part of IC-tester program.
 *  Copyright (C) 2001,2002  Silvio Klaic, sklaic@student.math.hr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "portlib.h"
#include "device.h"

void r_test( unsigned int IO )
{
char t[16];

 if (allow_io(IO, 2) == -1) {/* test port access */
  fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);
 }else{ /* all ok do read */
  if(is_power_on(IO)) printf("\n => WARNING: ictester device not powered - wrong reading output !\n\n");
  printf("Selected port is: 0x%03X, status at test socket on ictester:\n",IO);
  portread( IO, t );/* read status from device */
  printf("  .--. .--. .--. .--. .--. .--. .--. .--.\n\
  | %c| | %c| | %c| | %c| | %c| | %c| | %c| | %c|\n\
.-+--+-+--+-+--+-+--+-+--+-+--+-+--+-+--+-.\n\
|  16   15   14   13   12   11   10    9  |\n )\t\tTEST SOCKET\t\t  |\n\
| * 1    2    3    4    5    6    7    8  |\n\
`-+--+-+--+-+--+-+--+-+--+-+--+-+--+-+--+-'\n\
  | %c| | %c| | %c| | %c| | %c| | %c| | %c| | %c|\n\
  `--' `--' `--' `--' `--' `--' `--' `--'\n",t[15],t[14],t[13],t[12],t[11],
  t[10],t[9],t[8],t[0],t[1],t[2],t[3],t[4],t[5],t[6],t[7]);
  if (disallow_io(IO, 2) == -1)/* close port */
   fprintf(stderr, "Woha! Can't disengage port.\n\
You could have some problems with this port from now on...\n");
 }/* end else if port access */
}/* end r_test() */

void R_test( unsigned int IO )
{/* prapidly read from device */
char t[17],ot[17];
int i;

 if (allow_io(IO, 2) == -1) {/* test port access */
  fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);
 }else{ /* all ok do read */
  if(is_power_on(IO)) printf("\n => WARNING: ictester device not powered -\
 wrong reading output!\nAborting rapidly read...\n\n");
  else{/* display quick usage message and do test */
   printf("Rapidly read mode selected, use CTRL + C to abort reading.\n\
Selected port is: 0x%03X, status at test socket on ictester:\n\
Press ENTER key to begin read...",IO);getchar();
   printf("\nPIN No:\t01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16\n");
   for(i=0;i<=16;i++) ot[i]=0;/* clean compare */
   while(1){/* make endless loop */
    portread( IO, t );/* read status from device */
    t[16]=0;if(strcmp(t,ot)!=0){/* there changes in reading, display it */
     printf("Read:\t");for(i=0;i<=15;i++){/* copy data and display it */
      ot[i]=t[i];printf(" %c ",ot[i]);}/* end for i */
     printf("\n");}/* end if strcmp */
   }/* end while */
   if (disallow_io(IO, 2) == -1)/* close port */
    fprintf(stderr, "Woha! Can't disengage port.\n\
 You could have some problems with this port from now on...\n");
  }/* end else if device powered */
 }/* end else if port access */
}/* end R_test() */

void w_test( unsigned int IO, char *DAT )
{
int i;
char DATA[20];

 if(DAT==NULL) sprintf(DATA,"%-16s","="); /* no string make n/a to all */
 else{ DAT[16]=0;sprintf(DATA,"%-16s",DAT);}/* copy data or create new */
 if (allow_io(IO, 2) == -1) {/* test port access */
  fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);
 }else{ /* all ok do write */
  if(is_power_on(IO)) printf("\n => WARNING: ictester device not powered - wrong writing input !\n\n");
  for(i=0;i<=15;i++) if((DATA[i]!='1')&&(DATA[i]!='0')) DATA[i]='=';
  DATA[16]=0;/* replace characters and make string */
  printf("Selected port is: 0x%03X, writing [%16s] ... ",IO,DATA);
  enable_port(3,IO);/* enable ictester device */
  if(!portwrite(IO,DATA)) printf("Done.\n");/* write status */
  else printf("Fail!\nData aren't successful write to device.\n\
There is problem with transferring data to device, possible low power?\n");
  if (disallow_io(IO, 2) == -1)/* close port */
   fprintf(stderr, "Woha! Can't disengage port.\n\
You culd have some problems with this port from now on...\n");
 }/* end else if port access */
}/* end w_test() */

void lock_device(unsigned int IO)
{
if (allow_io(IO, 2) == -1) {/* test port access */
 fprintf(stderr,"Opa... You aren't being allowed access to the ports 0x%03X\
 and 0x%03X.\n",IO,IO+1);
}else{ /* all ok do write */
 printf("Selected port is: 0x%03X, locking and resetting ... ",IO);
 write_byte(32,IO);printf(" Done.\n");/* reset port and disable write */
 if (disallow_io(IO, 2) == -1)/* close port */
  fprintf(stderr, "Woha! Can't disengage port.\n\
You could have some problems with this port from now on...\n");
}/* end else if port access */
}/* end lock_device() */

